import { Request, Response } from 'express';
import { prisma, __path, log } from '../../../index.js';

// Get category.
function get(req: Request, res: Response) {
	// Check if required fields are present.
	if (!req.query.name) {
		res.status(400).json({ status: 'ERROR', errorcode: 'VALIDATION_ERROR', message: 'One or more required fields are missing' });
		return;
	}

	prisma.itemCategory
		.findUnique({
			where: {
				name: req.query.name.toString()
			}
		})
		.then((item) => {
			if (item) {
				res.status(200).json(item);
			} else {
				res.status(410).json({ status: 'ERROR', errorcode: 'NOT_EXISTING', message: 'Category does not exist' });
			}
		})
		.catch((err) => {
			log.db.error(err);
			res.status(500).json({ status: 'ERROR', errorcode: 'DB_ERROR', error: err, message: 'An error occurred during the database operation' });
		});
}

// Create category.
function post(req: Request, res: Response) {
	// Check if required fields are present.
	if (!req.body.name) {
		res.status(400).json({ status: 'ERROR', errorcode: 'VALIDATION_ERROR', message: 'One or more required fields are missing' });
		return;
	}

	// Save data.
	prisma.itemCategory
		.create({
			data: {
				name: req.body.name,
				description: req.body.description
			},
			select: {
				id: true
			}
		})
		.then((data) => {
			res.status(201).json({ status: 'CREATED', message: 'Successfully created category', id: data.id });
		})
		.catch((err) => {
			// Check if an entry already exists.
			if (err.code === 'P2002') {
				// P2002 -> "Unique constraint failed on the {constraint}"
				// https://www.prisma.io/docs/reference/api-reference/error-reference
				res.status(409).json({ status: 'ERROR', errorcode: 'EXISTING', message: 'Category already exists' });
			} else {
				log.db.error(err);
				res.status(500).json({ status: 'ERROR', errorcode: 'DB_ERROR', error: err, message: 'An error occurred during the database operation' });
			}
		});
}

// Update category.
async function patch(req: Request, res: Response) {
	// Check if required fields are present.
	if (!req.body.id || !req.body.name) {
		res.status(400).json({ status: 'ERROR', errorcode: 'VALIDATION_ERROR', message: 'One or more required fields are missing' });
		return;
	}

	// Check if the category id exists. If not return 410 Gone.
	try {
		const result = await prisma.itemCategory.findUnique({
			where: {
				id: parseInt(req.body.id)
			}
		});

		if (result === null) {
			res.status(410).json({ status: 'ERROR', errorcode: 'NOT_EXISTING', message: 'Category does not exist' });
			return;
		}
	} catch (err) {
		log.db.error(err);
		res.status(500).json({ status: 'ERROR', errorcode: 'DB_ERROR', error: err, message: 'An error occurred during the database operation' });
	}

	prisma.itemCategory
		.update({
			where: {
				id: parseInt(req.body.id)
			},
			data: {
				name: req.body.name,
				description: req.body.description
			},
			select: {
				id: true
			}
		})
		.then((data) => {
			res.status(201).json({ status: 'UPDATED', message: 'Successfully updated category', id: data.id });
		})
		.catch((err) => {
			// Check if an entry already exists.
			if (err.code === 'P2002') {
				// P2002 -> "Unique constraint failed on the {constraint}"
				// https://www.prisma.io/docs/reference/api-reference/error-reference
				res.status(409).json({ status: 'ERROR', errorcode: 'EXISTING', message: 'Category already exists' });
			} else {
				log.db.error(err);
				res.status(500).json({ status: 'ERROR', errorcode: 'DB_ERROR', error: err, message: 'An error occurred during the database operation' });
			}
		});
}

// Delete category.
async function del(req: Request, res: Response) {
	// Check if required fields are present.
	if (!req.body.id) {
		res.status(400).json({ status: 'ERROR', errorcode: 'VALIDATION_ERROR', message: 'One or more required fields are missing' });
		return;
	}

	// Does the id exist? If not return 410 Gone.
	try {
		const result = await prisma.itemCategory.findUnique({
			where: {
				id: parseInt(req.body.id)
			}
		});

		if (result === null) {
			res.status(410).json({ status: 'ERROR', errorcode: 'NOT_EXISTING', message: 'Category does not exist' });
			return;
		}
	} catch (err) {
		log.db.error(err);
		res.status(500).json({ status: 'ERROR', errorcode: 'DB_ERROR', error: err, message: 'An error occurred during the database operation' });
	}

	prisma.itemCategory
		.delete({
			where: {
				id: parseInt(req.body.id)
			}
		})
		.then(() => {
			res.status(200).json({ status: 'DELETED', message: 'Successfully deleted category' });
		})
		.catch((err) => {
			log.db.error(err);
			res.status(500).json({ status: 'ERROR', errorcode: 'DB_ERROR', error: err, message: 'An error occurred during the database operation' });
		});
}

export default { get, post, patch, del };
